require 'common.pingpong'
require 'common.shader'
require 'common.hsv'

glush.gui.checkbox('update',true)

fieldDim = 64

hsvtex = HsvTex.new(128)

function createFieldPingPong(args)
	args = args or {}
	return PingPong.new{
		width=fieldDim;
		height=fieldDim;
		internalFormat=gl.GL_RGBA_FLOAT32;
		format=gl.GL_RGBA;
		minFilter=gl.GL_NEAREST;
		magFilter=gl.GL_NEAREST;
		data=args.data;
	}
end

posTexs = createFieldPingPong{
-- [[
	data = function(u,v)
		return (u+.5)/fieldDim*2-1,(v+.5)/fieldDim*2-1, 0, 0
	end;
--]]
--[[
	data = function(u,v)
		local v = {math.random()*2-1, math.random()*2-1, math.random()*2-1}
		local l = v[1]*v[1] + v[2]*v[2] + v[3]*v[3]
		if l >= 1 then 
			v[1] = v[1] / l
			v[2] = v[2] / l
			v[3] = v[3] / l
		end
		return v[1], v[2], v[3], 1
	end;
--]]
}
velTexs = createFieldPingPong{data = function() return 0,0,0,0 end}

-- [[
vbo = gl.glGenBuffer()
gl.glBindBuffer(gl.GL_PIXEL_PACK_BUFFER, vbo)
gl.glBufferData(gl.GL_PIXEL_PACK_BUFFER, fieldDim * fieldDim * 4 * 4, nil, gl.GL_DYNAMIC_DRAW)
gl.glBindBuffer(gl.GL_PIXEL_PACK_BUFFER, 0)
--]]

dt = .001

simPosShader = KernelShader.new{
	texs={'postex', 'veltex'};
	code = [[
void main() {
	vec4 vpos = texture2D(postex, pos);
	vec4 vvel = texture2D(veltex, pos);
	const float dt = ]]..dt..[[;
	vpos.xyz += vvel.xyz * dt;
	vpos.w = length(vvel);
	gl_FragColor = vpos;
}
]];
}

simVelShader = KernelShader.new{
	texs={'postex','veltex'};
	code = [[
void main() {
	const float dt = ]]..dt..[[;
	const float gravConst = .1;
	vec4 vpos = texture2D(postex, pos);
	vec4 vvel = texture2D(veltex, pos);
	vec3 accumforce = vec3(0.);
	vec2 otc;
	for (otc.x = .5/]]..fieldDim..[[.; otc.x < 1.; otc.x += 1./]]..fieldDim..[[.) {
		for (otc.y = .5/]]..fieldDim..[[.; otc.y < 1.; otc.y += 1./]]..fieldDim..[[.) {
			vec4 otherpos = texture2D(postex, otc);
			vec3 del = otherpos.xyz - vpos.xyz;
			float len = length(del);
			len = max(len, .001);
			del *= (gravConst / len*len*len);
			accumforce += del;
		}
	}
	vvel.xyz += accumforce * dt;
	gl_FragColor = vvel;
}
]];
}

displayShader = ShaderProgram.new{
	uniforms={
		tex=0;
	};
	vertexCode = [[
varying float magn;
void main() {
	vec4 v = vec4(gl_Vertex.xyz, 1.);
	magn = gl_Vertex.w;
	gl_Position = gl_ModelViewProjectionMatrix * v;
}
]];
	fragmentCode = [[
#define	magnscale	.01
varying float magn;
uniform sampler1D tex;
void main() {
	gl_FragColor = texture1D(tex, magn * magnscale);
}
]];
}


gl.glDisable(gl.GL_DEPTH_TEST)
gl.glDisable(gl.GL_CULL_FACE)

function glush.display()
	gl.glEnable(gl.GL_TEXTURE_2D)
	
	local viewport = gl.glGet(gl.GL_VIEWPORT)
	gl.glViewport(0,0,fieldDim,fieldDim)
	gl.glMatrixMode(gl.GL_PROJECTION)
	gl.glPushMatrix()
	gl.glLoadIdentity()
	gl.glOrtho(0,1,0,1,-1,1)
	gl.glMatrixMode(gl.GL_MODELVIEW)
	gl.glPushMatrix()
	gl.glLoadIdentity()
	if update then
		velTexs:swap()
		velTexs:draw{
			shader=simVelShader;
			texs={posTexs:cur(), velTexs:prev()};
		}
		posTexs:swap()
		posTexs:draw{
			shader=simPosShader;
			texs={posTexs:prev(), velTexs:prev()};
		}
	end
	gl.glViewport(unpack(viewport))
	gl.glMatrixMode(gl.GL_PROJECTION)
	gl.glPopMatrix()
	gl.glMatrixMode(gl.GL_MODELVIEW)
	gl.glPopMatrix()

--[[
	velTexs:cur():bind()
	drawScreenQuad()
	velTexs:cur():unbind()
--]]
--[[
	gl.glPushMatrix()
	gl.glTranslate(-1,0,0)
	posTexs:cur():bind()
	drawScreenQuad()
	posTexs:cur():unbind()
	gl.glPopMatrix()
--]]


-- [[
	-- copy from texture to vbo
	gl.glBindBuffer(gl.GL_PIXEL_PACK_BUFFER, vbo)
	gl.glBindTexture(gl.GL_TEXTURE_2D, posTexs:cur().id)
	gl.glGetTexImage(gl.GL_TEXTURE_2D, 0, gl.GL_RGBA, gl.GL_FLOAT, 0)
	gl.glBindTexture(gl.GL_TEXTURE_2D, 0)
	gl.glBindBuffer(gl.GL_PIXEL_PACK_BUFFER, 0)

	gl.glPointSize(2)
	displayShader:use()

	gl.glEnableClientState(gl.GL_VERTEX_ARRAY)
	gl.glBindBuffer(gl.GL_ARRAY_BUFFER, vbo)
	gl.glVertexPointer(4, gl.GL_FLOAT, 0, 0)
	gl.glBindBuffer(gl.GL_ARRAY_BUFFER, 0)
	gl.glDrawArrays(gl.GL_POINTS, 0, fieldDim*fieldDim)
	gl.glDisableClientState(gl.GL_COLOR_ARRAY)

	gl.glUseProgramObject(0)
	gl.glPointSize(1)
--]]
end


